;; -----------------------------------------------------------------------
;;
;;   Copyright 2007 Dipl.-Ing. Michael Niederle - All Rights Reserved
;;
;;   This program is free software; you can redistribute it and/or modify
;;   it under the terms of the GNU General Public License as published by
;;   the Free Software Foundation, Inc., 53 Temple Place Ste 330,
;;   Boston MA 02111-1307, USA; either version 2 of the License, or
;;   (at your option) any later version; incorporated herein by reference.
;;
;; -----------------------------------------------------------------------

;;
;; ntloader.inc
;;
;; Common code for running a NT loader (ntldr or setupldr)
;;

;
; At this point, DX:AX contains the size of the kernel, and SI contains
; the file handle/cluster pointer.
;

; NT loader size must not exceed 384KB, because we're loading at 0x4000:0

is_ntloader:
                push si
                mov si, ntloader1_msg
                call cwritestr
                mov si, KernelCName
                call cwritestr
                mov si, ntloader2_msg
                call cwritestr
                pop si
 
;
; Now start transferring the NT loader
;
                push word real_mode_seg
                pop es

                movzx eax, ax			        ; Fix this by using a 32-bit
                shl edx, 16			          ; register for the kernel size
                or eax, edx
                cmp eax, 0x60000          ; maximum NT loader size
                jg .loader_too_big
                mov [KernelSize], eax
                add eax, SECTOR_SIZE-1
                shr eax, SECTOR_SHIFT
                mov [KernelSects], eax		; Total sectors in kernel
  
.next_page      mov ecx, 128              ; 1 page = 128 sectors
                cmp ecx, [KernelSects]
                jle .secnum_ok
                mov ecx, [KernelSects]
.secnum_ok      mov eax, ecx
                sub [KernelSects], ecx
                xor bx, bx
                call getfssec             ; read one page
                jc .load_failed
                mov ax, es
                add ax, 0x1000            ; next page
                mov es, ax
                cmp long [KernelSects], 0
                jg .next_page
                
.load_done:     push word real_mode_seg
                pop es
                mov eax, [es:0]
                mov dl, [DriveNumber]
                mov al, dl
                xor ecx, ecx
.wait:          dec ecx
                jne .wait
                mov si, superblock
                jmp word real_mode_seg:0  ; jump to NT loader main entry

.loader_too_big:
                mov si, err_loader_too_big
                jmp abort_load

.load_failed:
                mov si, err_load_failed
                jmp abort_load

ntloader1_msg	  db CR, LF, 'loading NT kernel "', 0
ntloader2_msg	  db '" ...', CR, LF, 0

err_loader_too_big:
                db CR, LF, 'NT loader size is too big!', CR, LF, 0

err_load_failed:
                db CR, LF, 'load failed!', CR, LF, 0

%include "writehex.inc"		; hexadecimal number output
